import os
import json
import random
import time
import requests
from tqdm import tqdm

# Load configuration settings
config_file = 'config.json'
if not os.path.exists(config_file):
    raise FileNotFoundError('Config file not found.')

with open(config_file, 'r', encoding='utf-8') as f:
    config = json.load(f)

if 'apiKeys' not in config or not isinstance(config['apiKeys'], list) or len(config['apiKeys']) == 0:
    raise ValueError('No API keys found in config.json')

api_key_index = 0
max_api_keys = len(config['apiKeys'])

def slugify(text, existing_slugs):
    """Generate a unique slug for the brand with hyphens for spaces."""
    base_slug = text.replace(" ", "-")  # Replace spaces with hyphens
    slug = base_slug
    counter = 1

    # Check for uniqueness
    while slug in existing_slugs:
        slug = f"{base_slug}-{counter}"  # Append counter to slug
        counter += 1

    return slug

def fetch_titles_and_descriptions(brand, pages_to_scrape):
    global api_key_index
    base_url = 'https://www.googleapis.com/customsearch/v1'
    search_results = []
    start = 1
    max_results = pages_to_scrape * 10

    while len(search_results) < max_results:
        api_key = config['apiKeys'][api_key_index]['key']
        cx = config['apiKeys'][api_key_index]['cx']
        url = f"{base_url}?key={api_key}&cx={cx}&q={brand}&start={start}"
        response = requests.get(url)

        if response.status_code != 200:
            api_key_index = (api_key_index + 1) % max_api_keys
            if api_key_index == 0:
                time.sleep(1)
            continue

        result = response.json()
        if 'items' not in result:
            break

        for item in result['items']:
            search_results.append({
                'title': item.get('title', 'No title'),
                'desc': item.get('snippet', 'No description')
            })

        start += 10

    return search_results

def get_random_image(assets_folder):
    image_files = [f for f in os.listdir(assets_folder) if f.lower().endswith(('.jpg', '.jpeg', '.png', '.gif'))]
    if not image_files:
        return ''
    return os.path.join(assets_folder, random.choice(image_files))

def save_data_to_json(data):
    base_file_name = 'web'
    file_index = 1
    while True:
        file_path = f"{base_file_name}_{file_index}.json"
        if os.path.exists(file_path) and os.path.getsize(file_path) > 20 * 1024 * 1024:
            file_index += 1
            continue

        existing_data = []
        if os.path.exists(file_path):
            with open(file_path, 'r', encoding='utf-8') as f:
                existing_data = json.load(f)

        existing_data.append(data)
        with open(file_path, 'w', encoding='utf-8') as f:
            json.dump(existing_data, f, ensure_ascii=False, indent=4)
        break

def load_processed_brands():
    try:
        with open('processed_brands.json', 'r', encoding='utf-8') as file:
            return json.load(file)
    except FileNotFoundError:
        return []

def save_processed_brands(processed_brands):
    with open('processed_brands.json', 'w', encoding='utf-8') as file:
        json.dump(processed_brands, file, ensure_ascii=False, indent=4)

def create_subfolders_and_files(brand, pages_to_scrape, processed_brands):
    if brand in processed_brands:
        return

    search_results = fetch_titles_and_descriptions(brand, pages_to_scrape)
    if not search_results:
        print(f"No results found for {brand}.")
        return

    descriptions = [result['desc'] for result in search_results]
    random.shuffle(descriptions)

    existing_slugs = set()  # To track existing slugs for uniqueness

    for result in search_results:
        slug = slugify(brand, existing_slugs)  # Generate a slug for the brand
        existing_slugs.add(slug)  # Add the slug to the set of existing slugs

        data_to_save = {
            'brand': brand,
            'slug': slug,  # Save the slug
            'title': result['title'],
            'desc': result['desc'],
            'desc2': random.choice(descriptions) if descriptions else '',
            'desc3': random.choice(descriptions) if descriptions else '',
            'harga_asli': random.randint(100000, 500000),
            'harga': random.randint(10000, 50000),
            'nilai': random.randint(1000, 9000000),
            'image': get_random_image(os.path.join(os.getcwd(), 'assets'))
        }
        save_data_to_json(data_to_save)

    processed_brands.append(brand)
    save_processed_brands(processed_brands)

# Main execution block
pages_to_scrape = 3
processed_brands = load_processed_brands()

with open('brand.txt', 'r', encoding='utf-8') as f:
    brands = [line.strip() for line in f.readlines() if line.strip()]
    brands = [brand for brand in brands if brand not in processed_brands]

random.shuffle(brands)

for brand in tqdm(brands, desc="Processing brands"):
    create_subfolders_and_files(brand, pages_to_scrape, processed_brands)

print("All operations completed! Data saved to multiple web.json files.")
